# This script demonstrates how to load AC data and display predictions from a stage 1 population CNN model
# from Pennington & David 2023

import numpy as np
import os
import matplotlib.pyplot as plt

from nems0 import get_setting
from nems0.recording import load_recording
from nems0 import xforms, preprocessing, modelspec, epoch


# CONFIGURATION

#  LOAD THE DATA
signals_dir = "file://"
modelspecs_dir = "file://"
cellids = ["DRX006b-128-2", "ARM030a-40-2"]

datafile = os.path.join(signals_dir, 'A1_NAT4_ozgf.fs100.ch18.tgz')
rec = load_recording(datafile)
all_cellids = rec['resp'].chans

# LOAD SOME MODEL FITS
modelnames = ['LN-pop','1Dx2-CNN']
modelspecfile_ln = os.path.join(modelspecs_dir, 'modelspec.ln_pop_a1.json')
modelspecfile_cnn = os.path.join(modelspecs_dir, 'modelspec.1dx2_cnn_a1.json')
model0 = modelspec.load_modelspec(modelspecfile_ln)
model1 = modelspec.load_modelspec(modelspecfile_cnn)

context = {'rec': rec}
context.update(xforms.normalize_sig(sig='stim', norm_method='minmax', log_compress=1, **context))
context.update(xforms.normalize_sig(sig='resp', norm_method='minmax', **context))
context.update(preprocessing.split_pop_rec_by_mask(**context))

val0 = model0.evaluate(rec=context['val'])
val1 = model1.evaluate(rec=context['val'])

stimulus_epochs = epoch.epoch_names_matching(val0['resp'].epochs, '^STIM_')
e = 1
t0 = 0
t1 = 450

stim = val0['stim'].as_continuous()
#resp = val0['resp'].extract_channels(cellids).extract_epoch(stimulus_epochs[e])[0,:,:]
resp = val0['resp'].extract_channels(cellids).as_continuous()
pred0 = val0['pred'].extract_channels(cellids).as_continuous()
pred1 = val1['pred'].extract_channels(cellids).as_continuous()

r_cell0_model0 = np.corrcoef(resp[0,:], pred0[0,:])[0,1]
r_cell0_model1 = np.corrcoef(resp[0,:], pred1[0,:])[0,1]
r_cell1_model0 = np.corrcoef(resp[1,:], pred0[1,:])[0,1]
r_cell1_model1 = np.corrcoef(resp[1,:], pred1[1,:])[0,1]

f, ax = plt.subplots(3, 1, sharex='col')

t=np.arange(t0,t1)/val0['resp'].fs

ax[0].imshow(stim[:,t0:t1], aspect='auto', origin='lower', interpolation='none', cmap='gray_r', extent=[t[0],t[-1],0.5,18.5])
ax[0].set_ylabel('Frequency channel')

ax[1].plot(t, resp[0,t0:t1], label='actual')
ax[1].plot(t, pred0[0,t0:t1], label=f"{modelnames[0]} (r={r_cell0_model0:.3f})")
ax[1].plot(t, pred1[0,t0:t1], label=f"{modelnames[1]} (r={r_cell0_model1:.3f})")
ax[1].set_title(cellids[0])
ax[1].set_ylabel('Spikes/s')
ax[1].legend()

ax[2].plot(t, resp[1,t0:t1], label='actual')
ax[2].plot(t, pred0[1,t0:t1], label=f"{modelnames[0]} (r={r_cell1_model0:.3f})")
ax[2].plot(t, pred1[1,t0:t1], label=f"{modelnames[1]} (r={r_cell1_model1:.3f})")
ax[2].set_title(cellids[1])
ax[2].set_xlabel('Time (s)')
ax[2].set_ylabel('Spikes/s')
ax[2].legend()

plt.tight_layout()
plt.suptitle('Comparison of predictions from stage 1 fit of 2 population CNN models')
